#!/usr/bin/env python
# -*- coding: utf-8 -*-
# R.Wobst, @(#) Feb 15 2012, 21:54:46

# module for wbd - help class

# help class - container for (filename, breakpoint_line, cond_lst, cmd_lst)

class Breakpoints:
    ## @brief c'tor - create empty tuple
    #
    # @param fn - filename

    def __init__(self, fn):
        self.lineno = -1
        self.filename = fn
        self.cond_lst = []
        self.cmd_lst = []
        self.cont = False       # True if last read line ended with "\"

    ## @brief add a condition
    #
    # @param cond - condition as string

    def addCond(self, cond):
        self._add2list(self.cond_lst, cond)

    ## @brief add a command - not used for python yet
    #
    # @param cmd - command as string

    def addCmd(self, cmd):
        self._add2list(self.cmd_lst, cmd)


    ## @brief collect possibly continued lines
    #
    # @param lst - list where conditions/commands are collected
    # @param line - read line, possibly continued

    def _add2list(self, lst, line):
        if self.cont:
            # continuation line, self.cond_lst must be non-empty
            lst[-1] += self._testcont(line)
        else:
            # plain append
            lst.append(self._testcont(line))

    ## @brief test on continuation mark "\" at line end
    #
    # If line ends with "\", replace it by blank and set self.cont to True.
    # Otherwise, set self.cont to False.
    #
    # @param line - read line
    # @return - possibly modified line

    def _testcont(self, line):
        if line and line[-1] == '\\':
            self.cont = True
            line = line[:-1] + ' '
        else:
            self.cont = False

        return line

## @brief proceed one file
#
# parameter bplst is possibly extended
#
# @param filename - name of file (relative or absolute path)
# @param comment - comment sequence (# or //)
# @param token - token for debug description section, e.g. DEB or TEST
# @param bplst - list of wbdhelp.Breakpoints instances

def proc_file(filename, comment, token, bplst):
    fi = open(filename, 'r')

    lno = 0
    bp = None
    cl = len(comment)
    cmd = False

    for line in fi:
        lno += 1
        if bp:
            line = line.strip()
            if line.startswith(comment):
                if line[cl:] == 'CMD':
                    cmd = True
                    bp.addCmd('commands')
                    continue
                if cmd:
                    bp.addCmd(line[cl:])
                    continue
                # no cmd, must be condition
                bp.addCond(line[cl:])
            else:
                # no more conditions
                if (bp.cond_lst and bp.cond_lst[-1][-1] == '\\') or \
                        (bp.cmd_lst and bp.cmd_lst[-1][-1] == '\\'):
                    print "unclosed continuation: file %s, line %d" % \
                            (filename, lno)
                    import sys
                    sys.exit(1)

                bp.lineno = lno         # not consequent
                bplst.append(bp)
                bp = None
                cmd = False

        if line.strip()[cl:] == token:
            bp = Breakpoints(filename)

    fi.close()
